#! /usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright(C) 2013 Mark Tully <markjtully@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import GLib, Gio
from gi.repository import Unity
import gettext
import os
import sqlite3

APP_NAME = 'unity-scope-gourmet'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Recipes.Gourmet'
UNIQUE_PATH = '/com/canonical/unity/scope/recipes/gourmet'

SEARCH_HINT = _('Search Gourmet Recipe Manager')
NO_RESULTS_HINT = _('Sorry, there are no Gourmet recipe results that match your search.')
PROVIDER_CREDITS = _('')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR + 'service-yelp.svg'
DEFAULT_RESULT_ICON = SVG_DIR + 'result-help.svg'
DEFAULT_RESULT_MIMETYPE = 'x-scheme-handler/man'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT
GOURMET_DB = os.getenv("HOME") + '/.gourmet/recipes.db'

STARS = {0: '☆☆☆☆☆',
         1: '✮☆☆☆☆',
         2: '★☆☆☆☆',
         3: '★✮☆☆☆',
         4: '★★☆☆☆',
         5: '★★✮☆☆',
         6: '★★★☆☆',
         7: '★★★✮☆',
         8: '★★★★☆',
         9: '★★★★✮',
         10: '★★★★★'}

c1 = {'id': 'recipes',
      'name': _('Recipes'),
      'icon': SVG_DIR + 'group-installed.svg',
      'renderer': Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [c1]

FILTERS = []

EXTRA_METADATA = []


def duration(seconds):
    if isinstance(seconds, int):
        import time
        if seconds > 3600:
            duration = time.strftime(_('%H hours %M minutes'), time.gmtime(seconds))
        else:
            duration = time.strftime(_('%M minutes'), time.gmtime(seconds))
        return duration
    else:
        return ""


def gourmet_search(path, search):
    try:
        conn = sqlite3.connect(path)
        cursor = conn.cursor()
        print("Searching for Gourmet recipes")
        cursor.execute('SELECT * FROM recipe WHERE title LIKE ?', ('%' + search + '%', ))
        results = cursor.fetchall()
        cursor.close()
        return results
    except:
        return []


def search(search, filters):
    '''
    Search for help documents matching the search string
    '''
    results = []
    if not os.path.exists('/tmp/unity-scope-gourmet'):
        os.mkdir('/tmp/unity-scope-gourmet')

    gourmet_results = gourmet_search(GOURMET_DB, search)
    for row in gourmet_results:
        # Give each cached image file the unique ID of its recipe
        if not (row[15]):
            i = row[0]
            title = row[1]
            uri = row[18]
            likes = row[5]
            likes = STARS[likes]

            comments = row[3]
            time = row[8] + row[9]

            if(time):
                time = str(duration(time))
                comment = '%s|%s\n%s' % (likes, time, str(comments))
            else:
                comment = '%s\n%s' % (likes, str(comments))

            # add http:// if necessary
            if not (uri.startswith("http://")) and not (uri == ''):
                uri = 'http://%s' % uri

            if row[14]:
                open('/tmp/unity-scope-gourmet/icon' + str(i), 'wb').write(row[14])
                icon_hint = '/tmp/unity-scope-gourmet/icon%s' % str(i)
            else:
                if os.path.exists('/tmp/unity-scope-gourmet/icon' + str(i)):
                    os.remove('/tmp/unity-scope-gourmet/icon' + str(i))
            icon_hint = '/usr/share/unity/lenses/cooking/icons/cuisine.png'

            results.append({'uri': uri,
                            'icon': icon_hint,
                            'category': 0,
                            'title': title,
                            'comment': comment})
    return results


# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch(Unity.ScopeSearchBase):
    def __init__(self, search_context):
        super(MySearch, self).__init__()
        self.set_search_context(search_context)

    def do_run(self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                i['metadata'] = {}
                if EXTRA_METADATA:
                    for e in i:
                        for m in EXTRA_METADATA:
                            if m['id'] == e:
                                i['metadata'][e] = i[e]
                i['metadata']['provider_credits'] = GLib.Variant('s', PROVIDER_CREDITS)
                result = Unity.ScopeResult.create(str(i['uri']), str(i['icon']),
                                                  i['category'], i['result_type'],
                                                  str(i['mimetype']), str(i['title']),
                                                  str(i['comment']), str(i['dnd_uri']),
                                                  i['metadata'])
                result_set.add_result(result)
        except Exception as error:
            print(error)


class Scope(Unity.AbstractScope):
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint(self):
        return SEARCH_HINT

    def do_get_schema(self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new()
        if EXTRA_METADATA:
            for m in EXTRA_METADATA:
                schema.add_field(m['id'], m['type'], m['field'])
        #FIXME should be REQUIRED for credits
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories(self):
        '''
        Adds categories
        '''
        cs = Unity.CategorySet.new()
        if CATEGORIES:
            for c in CATEGORIES:
                cat = Unity.Category.new(c['id'], c['name'],
                                         Gio.ThemedIcon.new(c['icon']),
                                         c['renderer'])
                cs.add(cat)
        return cs

    def do_get_filters(self):
        '''
        Adds filters
        '''
        fs = Unity.FilterSet.new()
        #if FILTERS:
        #
        return fs

    def do_get_group_name(self):
        return GROUP_NAME

    def do_get_unique_name(self):
        return UNIQUE_PATH

    def do_create_search_for_query(self, search_context):
        se = MySearch(search_context)
        return se


def load_scope():
    return Scope()
